import { hash } from "ohash";
import { GqlOperations, GqlInstance } from "#build/gql";
import { extractOperation } from "ohmygql/utils";
import { gqlSdk } from "#build/gql-sdk";
import { useState, useNuxtApp, useAsyncData, useRuntimeConfig } from "#imports";
const useGqlState = () => {
  const nuxtApp = useNuxtApp();
  return nuxtApp?._gqlState;
};
const setGqlState = ({ client = "default", patch }) => {
  const state = useGqlState();
  state.value[client].instance.setOptions(patch);
};
export function useGqlHeaders(...args) {
  const client = args[1] || args?.[0]?.client;
  const headers = args[0] && typeof args[0] !== "undefined" && "headers" in args[0] ? args[0].headers : args[0];
  const respectDefaults = args?.[0]?.respectDefaults;
  setGqlState({ client, patch: { headers } });
  if (respectDefaults && !Object.keys(headers).length) {
    const defaultHeaders = useRuntimeConfig()?.public?.["graphql-client"]?.clients?.[client || "default"]?.headers;
    setGqlState({ client, patch: { headers: defaultHeaders } });
  }
}
const DEFAULT_AUTH = { type: "Bearer", name: "Authorization" };
export function useGqlToken(...args) {
  args = args || [];
  const token = typeof args[0] === "string" ? args[0] : args?.[0]?.token;
  const client = args[0]?.client || args?.[1]?.client;
  let config = args[0]?.config || args?.[1]?.config;
  const clientConfig = useRuntimeConfig()?.public?.["graphql-client"]?.clients?.[client || "default"];
  config = {
    ...DEFAULT_AUTH,
    ...clientConfig?.token?.name && { name: clientConfig.token.name },
    ...clientConfig?.token?.type !== void 0 && { type: clientConfig.token.type },
    ...config
  };
  setGqlState({
    client,
    patch: { headers: { [config.name]: !token ? void 0 : `${config.type} ${token}`.trim() } }
  });
}
export const useGqlCors = (cors) => {
  const { mode, credentials, client } = cors || {};
  setGqlState({ client, patch: { mode, credentials } });
};
export const useGql = () => {
  const state = useGqlState();
  const errState = useGqlErrorState();
  const handle = (client) => {
    client = client || "default";
    const { instance } = state.value?.[client];
    instance.setMiddleware({
      onResponseError: async ({ options, response }) => {
        errState.value = {
          client,
          status: response?.status,
          operation: extractOperation(JSON.parse(options?.body)?.query || ""),
          gqlErrors: Array.isArray(response?._data?.errors) ? response?._data?.errors : [response?._data]
        };
        if (state.value.onError) {
          state.value.onError(errState.value);
        }
      }
    });
    return gqlSdk(instance);
  };
  return { handle };
};
export const useGqlError = (onError) => {
  useGqlState().value.onError = process.client ? onError : process.env.NODE_ENV !== "production" && ((e) => console.error("[nuxt-graphql-client] [GraphQL error]", e));
  const errState = useGqlErrorState();
  if (!errState.value) {
    return;
  }
  onError(errState.value);
};
const useGqlErrorState = () => useState("_gqlErrors", () => null);
export function useAsyncGql(...args) {
  const operation = (typeof args?.[0] !== "string" && "operation" in args?.[0] ? args[0].operation : args[0]) ?? void 0;
  const variables = (typeof args?.[0] !== "string" && "variables" in args?.[0] ? args[0].variables : args[1]) ?? void 0;
  const options = (typeof args?.[0] !== "string" && "options" in args?.[0] ? args[0].options : args[2]) ?? void 0;
  const client = Object.keys(GqlOperations).find((k) => GqlOperations[k].includes(operation)) ?? "default";
  const key = hash({ operation, client, variables });
  return useAsyncData(key, () => GqlInstance().handle(client)[operation](variables), options);
}
